/**
 * 
 */
package com.hdp.pi.service.kolon;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import au.com.bytecode.opencsv.CSVWriter;

import com.hdp.pi.domain.kolon.KolonMember;
import com.hdp.pi.dto.kolon.KolonMemberDTO;
import com.hdp.pi.repository.kolon.KolonMemberPreassignSnRepository;
import com.hdp.pi.repository.kolon.KolonMemberRepository;
import com.hdp.pi.utils.kolon.FtpUtil;
import com.hdp.pi.utils.kolon.Util;

/**
 * @author yangyw
 *
 */
public class KolonMemberServiceImpl implements KolonMemberService {
	
	@Autowired
	private FtpUtil ftpUtil;

	@Autowired
	private KolonMemberRepository kolonMemberRepository;

	@Autowired
	private KolonMemberPreassignSnRepository kolonMemberPreassignSnRepository;
	
	@Override
	public KolonMember findOneByCsNo(Long csNo) {
		return kolonMemberRepository.findFirstByCsNo(csNo);
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
	public void save(KolonMember kolonMember) {
		kolonMemberRepository.save(kolonMember);
	}

	@Override
	public List<KolonMember> findByUpdateTimeGreaterThan(Date updateTime) {
		return kolonMemberRepository.findByUpdateTimeGreaterThan(updateTime);
	}

	@Override
	public List<KolonMember> findByDataSourceAndUpdateTimeGreaterThan(
			Integer dataSource, Date updateTime) {
		return kolonMemberRepository.findByDataSourceAndUpdateTimeGreaterThan(
				dataSource, updateTime);
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
	public void matchData(String[] s) {
		KolonMemberDTO kolonMemberDTO = new KolonMemberDTO(s);
		Long csNo = kolonMemberDTO.csNo;
		KolonMember kolonMember = null;
		if(StringUtils.isNotEmpty(kolonMemberDTO.phone)){
			kolonMember = kolonMemberRepository.findFirstByPhone(kolonMemberDTO.phone);
		}
		if(kolonMember == null){
			kolonMember = this.findOneByCsNo(csNo);
		}
		if (kolonMember == null) { // 插入数据
			kolonMember = new KolonMember();
			kolonMember.csNo = csNo;
			kolonMember.phone = kolonMemberDTO.phone;
			kolonMember.lastName = kolonMemberDTO.lastName;
			kolonMember.firstName = kolonMemberDTO.firstName;
			kolonMember.email = kolonMemberDTO.email;
			kolonMember.status = kolonMemberDTO.status;
			kolonMember.bonus = kolonMemberDTO.bonus;
			kolonMember.lifecycle = kolonMemberDTO.lifecycle;
			kolonMember.customerType = kolonMemberDTO.customerType;
			kolonMember.dataSource = 2;
			Date date = new Date();
			kolonMember.joinTime = date;
			kolonMember.updateTime = date;
			this.save(kolonMember);
		} else { // 更新
			boolean flag = false;
			Long oldCsNo = null;//是否更新了csno
			if(!kolonMemberDTO.csNo.equals(kolonMember.csNo)){//kolon和派加都注册了，以kolon为准
				oldCsNo = kolonMember.csNo;
				kolonMember.csNo = kolonMemberDTO.csNo;
				flag = true;
				
			}
			if(!kolonMemberDTO.cardNo.equals(kolonMember.cardNo)){//kolon和派加都注册了，以kolon为准
				kolonMember.cardNo = kolonMemberDTO.cardNo;
				flag = true;
			}
			
			if (!Util.isNotNull(kolonMember.email)
					|| !kolonMember.email.endsWith(kolonMemberDTO.email)) {
				kolonMember.email = kolonMemberDTO.email;
				flag = true;
			}
			if (!Util.isNotNull(kolonMember.firstName)
					|| !kolonMember.firstName
							.endsWith(kolonMemberDTO.firstName)) {
				kolonMember.firstName = kolonMemberDTO.firstName;
				flag = true;
			}
			if (!Util.isNotNull(kolonMember.lastName)
					|| !kolonMember.lastName
							.endsWith(kolonMemberDTO.lastName)) {
				kolonMember.lastName = kolonMemberDTO.lastName;
				flag = true;
			}
			if (flag) {
				kolonMember.updateTime = new Date();
				this.save(kolonMember);
				if(oldCsNo!=null){//csno被kolon覆盖
					kolonMemberPreassignSnRepository.setStatusWithKolonReplace(oldCsNo);
				}
			}
		}
	}

	@Override
	public void syncData() {
		List<String[]> list = ftpUtil.readCSVFile();
		if (list != null) {
			boolean firstRow = true;
			for (String[] row : list) {
				if (firstRow) {
					firstRow = false;
				} else {
					this.matchData(row);
				}
			}
		}
		ftpUtil.moveFile();
	}

	
	@Override
	public Date sendData(Date time) {
		Date lastSendDate = time;
		
		//获取记录
		List<KolonMember> kolonMembers = kolonMemberRepository.findByJoinTimeGreaterThanOrUpdateTimeGreaterThan(time, time);
		
		List<String[]> addData = getDataList();
		List<String[]> updateData = getDataList();
		
		for(KolonMember km : kolonMembers){
			if(km.dataSource == 2){
				updateData.add(new String[] { km.csNo.toString(), km.phone, km.email, km.wechatFirstName, km.wechatLastName});
			}else if(km.dataSource == 1 && km.joinTime.before(km.updateTime)){  //加入时间在更新时间之前，数据是更新的
				updateData.add(new String[] { km.csNo.toString(), km.phone, km.email, km.wechatFirstName, km.wechatLastName});
			}else{
				addData.add(new String[] { km.csNo.toString(), km.phone, km.email, km.wechatFirstName, km.wechatLastName});
			}
			if(lastSendDate.before(km.updateTime)){  //获取数据较后的更新时间
				lastSendDate = km.updateTime;
			}
		}
		
		CSVWriter addWriter = ftpUtil.getCSVWriter(1);
		writeDataToFile(addWriter, addData);
		CSVWriter updateWriter = ftpUtil.getCSVWriter(2);
		writeDataToFile(updateWriter, updateData);
		
		//上传文件
		uploadFile();
		return lastSendDate;
	}

	private List<String[]> getDataList() {
		List<String[]> addData = new ArrayList<String[]>();
		String[] header = "CS_NO,Phone,Email,Wechat Last Name,Wechat First Name".split(",");
		addData.add(header);
		return addData;
	}
	
	public void writeDataToFile(CSVWriter writer, List<String[]> data){
		try {
			writer.writeAll(data);
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public void uploadFile(){
		ftpUtil.uploadFile();
	}

}
